==> mdk-0.1.1.tar.gz: mdk-0.1.1/mixlib/mix_device.c <==
/* -*-c-*- -------------- mix_device.c :
 * Implementation of the functions declared in mix_device.h
 * ------------------------------------------------------------------
 * Copyright (C) 2000 jose antonio ortega ruiz <jaortega@acm.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 */


#include "mix_file.h"
#include "mix_device.h"

/*
  Actual definition of a mix device, which can be cast to
  a mix file.
*/
struct mix_device_t 
{
  mix_iochannel_t *file;
  mix_device_type_t type;
};


static const char *DEV_EXT_ = ".dev";

static const char *DEF_NAMES_[] = {
  "tape0", "tape1", "tape2", "tape3", "tape4", "tape5", "tape6", "tape7",
  "disk0", "disk1", "disk2", "disk3", "disk4", "disk5", "disk6", "disk7",
  "cardrd", "cardwr", "printer", "console", "paper"
};

static const size_t SIZES_[] = {
  100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100,
  16, 16, 24, 14, 14
};

static const mix_device_mode_t MODES_[] = {
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,  
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,  
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,
  mix_dev_BIN, mix_dev_CHAR, mix_dev_CHAR, mix_dev_CHAR, mix_dev_CHAR,
  mix_dev_CHAR
};

static const mix_fmode_t FMODES_[] = {
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_READ, mix_io_WRITE, mix_io_WRITE, mix_io_RDWRT, mix_io_WRITE
};

#define GET_CHANNEL_(dev) (dev->file)
#define GET_FILE_(dev) ((mix_file_t *)(dev->file))

mix_device_t *
mix_device_new (mix_device_type_t type)
{
  mix_device_t *result = g_new (mix_device_t, 1);
  g_return_val_if_fail (result != NULL, result);
  result->type = type;
  if (type != mix_dev_CONSOLE) {
    result->file = MIX_IOCHANNEL(mix_file_new_with_def_ext (DEF_NAMES_[type],
							    FMODES_[type],
							    DEV_EXT_));
  } else
    result->file = mix_io_new (stdout);
    
  return result;
}

void
mix_device_delete (mix_device_t *dev)
{
  if (dev != NULL)
    {
      if (dev->type != mix_dev_CONSOLE && GET_FILE_(dev) != NULL)
	mix_file_delete (GET_FILE_(dev));
      g_free (dev);
    }
}

mix_device_t *
mix_device_new_with_name (mix_device_type_t type, const gchar *name)
{
  mix_device_t *result = NULL;
  g_return_val_if_fail (name != NULL, result);
  result = g_new (mix_device_t, 1);  
  g_return_val_if_fail (result != NULL, result);
  result->type = type;
  if (type != mix_dev_CONSOLE)
    {
      result->file = MIX_IOCHANNEL(mix_file_new_with_def_ext (name,
							      FMODES_[type],
							      DEV_EXT_));
    }
  else
    {
      result->file = mix_io_new (stdout);
    }
  return result;
}

const char *
mix_device_get_name (mix_device_t *dev)
{
  g_return_val_if_fail (dev != NULL, NULL);
  return mix_file_base_name(GET_FILE_(dev));
}

/*
  Get the device block size
*/
size_t
mix_device_block_size (mix_device_t *dev)
{
  g_return_val_if_fail (dev != NULL, 0);
  return SIZES_[dev->type];
}

/*
  Get the device io mode
*/
mix_device_mode_t
mix_device_mode (mix_device_t *dev) 
{
  g_return_val_if_fail (dev != NULL, 0);
  return MODES_[dev->type];
}

/*
  Write a block to the device.
*/
gboolean
mix_device_write (mix_device_t *dev, const mix_word_t *block) 
{
  gboolean result;
  
  g_return_val_if_fail (dev != NULL, FALSE);
  g_return_val_if_fail (block != NULL, FALSE);
  if (FMODES_[dev->type] == mix_io_READ) return FALSE;
  if (MODES_[dev->type] == mix_dev_CHAR)
    result = mix_io_write_word_array_as_char (GET_CHANNEL_ (dev),
					      block, SIZES_[dev->type]);
  else
    result = mix_io_write_word_array (GET_CHANNEL_ (dev),
				      block, SIZES_[dev->type]);
  if (result && mix_device_mode(dev) == mix_dev_CHAR) 
    putc ('\n', mix_io_to_FILE (GET_CHANNEL_ (dev)));
  
  return result;
}

gboolean
mix_device_read (mix_device_t *dev, mix_word_t *block) 
{
  g_return_val_if_fail (dev != NULL, FALSE);
  g_return_val_if_fail (block != NULL, FALSE);
  if (FMODES_[dev->type] == mix_io_WRITE) return FALSE;
  if (MODES_[dev->type] == mix_dev_CHAR)
    return mix_io_read_word_array_as_char (GET_CHANNEL_ (dev),
					   block, SIZES_[dev->type]);
  else
    return mix_io_read_word_array (GET_CHANNEL_ (dev),
				   block, SIZES_[dev->type]);
}

gboolean
mix_device_ioc (mix_device_t *dev, mix_short_t arg) 
{
  int m;
  FILE *file;
  
  g_return_val_if_fail (dev != NULL, FALSE);

  m = mix_short_magnitude(arg);
  if (mix_short_is_negative(arg)) m = -m;
  m *= sizeof (mix_word_t) * SIZES_[dev->type];
  file = mix_io_to_FILE (GET_CHANNEL_(dev));
  
  if (dev->type >= mix_dev_TAPE_0 && dev->type <= mix_dev_TAPE_7)
    {
      if (m == 0) rewind (file);
      else fseek (file, m, SEEK_CUR);
    }
  if (dev->type >= mix_dev_DISK_0 && dev->type <= mix_dev_DISK_7)
    {
      g_return_val_if_fail (m == 0, FALSE);
      // position disk
    }
  if (dev->type == mix_dev_PAPER_TAPE)
    {
      g_return_val_if_fail (m == 0, FALSE);
      rewind (file);
    }
  return TRUE;
}

gboolean
mix_device_busy (mix_device_t *dev)
{
  return (dev != NULL && !mix_io_is_ready(GET_CHANNEL_(dev)));
}

==> mdk-0.2.tar.gz: mdk-0.2/mixlib/mix_device.c <==
/* -*-c-*- -------------- mix_device.c :
 * Implementation of the functions declared in mix_device.h
 * ------------------------------------------------------------------
 * Copyright (C) 2000 jose antonio ortega ruiz <jaortega@acm.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 */


#include "mix_file.h"
#include "mix_device.h"

/*
  Actual definition of a mix device, which can be cast to
  a mix file.
*/
struct mix_device_t 
{
  mix_iochannel_t *file;
  mix_device_type_t type;
};


static const char *DEV_EXT_ = ".dev";

static const char *DEF_NAMES_[] = {
  "tape0", "tape1", "tape2", "tape3", "tape4", "tape5", "tape6", "tape7",
  "disk0", "disk1", "disk2", "disk3", "disk4", "disk5", "disk6", "disk7",
  "cardrd", "cardwr", "printer", "console", "paper"
};

static const size_t SIZES_[] = {
  100, 100, 100, 100, 100, 100, 100, 100,
  100, 100, 100, 100, 100, 100, 100, 100,
  16, 16, 24, 14, 14
};

static const mix_device_mode_t MODES_[] = {
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,  
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,  
  mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN, mix_dev_BIN,
  mix_dev_BIN, mix_dev_CHAR, mix_dev_CHAR, mix_dev_CHAR, mix_dev_CHAR,
  mix_dev_CHAR
};

static const mix_fmode_t FMODES_[] = {
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,  mix_io_RDWRT,
  mix_io_READ, mix_io_WRITE, mix_io_WRITE, mix_io_RDWRT, mix_io_WRITE
};

#define GET_CHANNEL_(dev) (dev->file)
#define GET_FILE_(dev) ((mix_file_t *)(dev->file))

mix_device_t *
mix_device_new (mix_device_type_t type)
{
  mix_device_t *result = g_new (mix_device_t, 1);
  result->type = type;
  if (type != mix_dev_CONSOLE) {
    result->file = MIX_IOCHANNEL(mix_file_new_with_def_ext (DEF_NAMES_[type],
							    FMODES_[type],
							    DEV_EXT_));
  } else
    result->file = mix_io_new (stdout);
    
  return result;
}

void
mix_device_delete (mix_device_t *dev)
{
  if (dev != NULL)
    {
      if (dev->type != mix_dev_CONSOLE && GET_FILE_(dev) != NULL)
	mix_file_delete (GET_FILE_(dev));
      g_free (dev);
    }
}

mix_device_t *
mix_device_new_with_name (mix_device_type_t type, const gchar *name)
{
  mix_device_t *result = NULL;
  g_return_val_if_fail (name != NULL, result);
  result = g_new (mix_device_t, 1);  
  result->type = type;
  if (type != mix_dev_CONSOLE)
    {
      result->file = MIX_IOCHANNEL(mix_file_new_with_def_ext (name,
							      FMODES_[type],
							      DEV_EXT_));
    }
  else
    {
      result->file = mix_io_new (stdout);
    }
  return result;
}

const char *
mix_device_get_name (mix_device_t *dev)
{
  g_return_val_if_fail (dev != NULL, NULL);
  return mix_file_base_name(GET_FILE_(dev));
}

/*
  Get the device block size
*/
size_t
mix_device_block_size (mix_device_t *dev)
{
  g_return_val_if_fail (dev != NULL, 0);
  return SIZES_[dev->type];
}

/*
  Get the device io mode
*/
mix_device_mode_t
mix_device_mode (mix_device_t *dev) 
{
  g_return_val_if_fail (dev != NULL, 0);
  return MODES_[dev->type];
}

/*
  Write a block to the device.
*/
gboolean
mix_device_write (mix_device_t *dev, const mix_word_t *block) 
{
  gboolean result;
  
  g_return_val_if_fail (dev != NULL, FALSE);
  g_return_val_if_fail (block != NULL, FALSE);
  if (FMODES_[dev->type] == mix_io_READ) return FALSE;
  if (MODES_[dev->type] == mix_dev_CHAR)
    result = mix_io_write_word_array_as_char (GET_CHANNEL_ (dev),
					      block, SIZES_[dev->type]);
  else
    result = mix_io_write_word_array (GET_CHANNEL_ (dev),
				      block, SIZES_[dev->type]);
  if (result && mix_device_mode(dev) == mix_dev_CHAR) 
    putc ('\n', mix_io_to_FILE (GET_CHANNEL_ (dev)));

  fflush (mix_io_to_FILE (GET_CHANNEL_ (dev)));
  
  return result;
}

gboolean
mix_device_read (mix_device_t *dev, mix_word_t *block) 
{
  g_return_val_if_fail (dev != NULL, FALSE);
  g_return_val_if_fail (block != NULL, FALSE);
  if (FMODES_[dev->type] == mix_io_WRITE) return FALSE;
  if (MODES_[dev->type] == mix_dev_CHAR)
    return mix_io_read_word_array_as_char (GET_CHANNEL_ (dev),
					   block, SIZES_[dev->type]);
  else
    return mix_io_read_word_array (GET_CHANNEL_ (dev),
				   block, SIZES_[dev->type]);
}

gboolean
mix_device_ioc (mix_device_t *dev, mix_short_t arg) 
{
  int m;
  FILE *file;
  
  g_return_val_if_fail (dev != NULL, FALSE);

  m = mix_short_magnitude(arg);
  if (mix_short_is_negative(arg)) m = -m;
  m *= sizeof (mix_word_t) * SIZES_[dev->type];
  file = mix_io_to_FILE (GET_CHANNEL_(dev));
  
  if (dev->type >= mix_dev_TAPE_0 && dev->type <= mix_dev_TAPE_7)
    {
      if (m == 0) rewind (file);
      else fseek (file, m, SEEK_CUR);
    }
  if (dev->type >= mix_dev_DISK_0 && dev->type <= mix_dev_DISK_7)
    {
      g_return_val_if_fail (m == 0, FALSE);
      // position disk
    }
  if (dev->type == mix_dev_PAPER_TAPE)
    {
      g_return_val_if_fail (m == 0, FALSE);
      rewind (file);
    }
  return TRUE;
}

gboolean
mix_device_busy (mix_device_t *dev)
{
  return (dev != NULL && !mix_io_is_ready(GET_CHANNEL_(dev)));
}
